==> pngme-api-0.9.6.tar.gz: pngme-api-0.9.6/pngme/api/resources/users.py <==
import asyncio
from abc import ABC
from datetime import datetime
from typing import Any, Dict, List, Optional

from pydantic import validate_arguments

from ..cache import cache
from ..core import BaseClient
from ..encoders import encode_query_params
from .errors import ServerError
from .retry import retry_policy

PATH = "/users"

User = Dict[str, Any]
UsersMeta = Dict[str, Any]
UsersResponse = Dict[str, Any]


class BaseUsersResource(ABC):
    def __init__(self, client: BaseClient):
        self._client = client

    @retry_policy(
        max_retry_count=4,
        max_retry_interval_seconds=10,
        retryable_exceptions=[ServerError],
    )
    async def _get_page(
        self,
        created_after: Optional[datetime] = None,
        created_before: Optional[datetime] = None,
        search: Optional[str] = None,
        page: int = 1,
    ) -> UsersResponse:
        async with self._client.session() as session:
            response = await session.get(
                PATH,
                params=encode_query_params(
                    created_after=created_after,
                    created_before=created_before,
                    search=search,
                    page=page,
                ),
            )

        if response.status_code == 500:
            raise ServerError(response.text)

        assert response.status_code == 200, response.text
        return response.json()  # type: ignore

    @validate_arguments
    @cache
    async def _get(
        self,
        created_after: Optional[datetime] = None,
        created_before: Optional[datetime] = None,
        search: Optional[str] = None,
    ) -> List[User]:
        response = await self._get_page(
            created_after=created_after,
            created_before=created_before,
            search=search,
        )
        max_pages = response["_meta"]["max_pages"]
        if max_pages > 1:
            coroutines = [
                self._get_page(
                    created_after=created_after,
                    created_before=created_before,
                    search=search,
                    page=page + 2,
                )
                for page in range(max_pages - 1)
            ]
            response_pages = await asyncio.gather(*coroutines)
            responses = (response, *response_pages)
        else:
            responses = (response,)

        return [user for response in responses for user in response["users"]]


class AsyncUsersResource(BaseUsersResource):
    async def get(
        self,
        *,
        created_after: Optional[datetime] = None,
        created_before: Optional[datetime] = None,
        search: Optional[str] = None,
    ) -> List[User]:
        return await self._get(
            created_after=created_after,
            created_before=created_before,
            search=search,
        )


class SyncUsersResource(BaseUsersResource):
    def get(
        self,
        *,
        created_after: Optional[datetime] = None,
        created_before: Optional[datetime] = None,
        search: Optional[str] = None,
    ) -> List[User]:
        return self._client.run(
            self._get(
                created_after=created_after,
                created_before=created_before,
                search=search,
            )
        )

==> pngme-api-0.10.1.tar.gz: pngme-api-0.10.1/pngme/api/resources/users.py <==
import asyncio
from abc import ABC
from datetime import datetime
from typing import Any, Dict, List, Optional, TypedDict

from pydantic import validate_arguments

from ..cache import cache
from ..core import BaseClient
from ..encoders import encode_query_params
from ..errors import ServerError, UserNotFoundError
from ..retry import retry_policy

PATH = "/users"
USER_PATH = "/users/{user_uuid}"

User = Dict[str, Any]


class UsersResponse(TypedDict):
    page: int
    num_pages: int
    users: List[User]


class BaseUsersResource(ABC):
    def __init__(self, client: BaseClient):
        self._client = client

    @retry_policy(
        max_retry_count=4,
        max_retry_interval_seconds=10,
        retryable_exceptions=[ServerError],
    )
    async def _get_page(
        self,
        created_after: Optional[datetime] = None,
        created_before: Optional[datetime] = None,
        search: Optional[str] = None,
        page: int = 1,
    ) -> UsersResponse:
        async with self._client.session() as session:
            response = await session.get(
                PATH,
                params=encode_query_params(
                    created_after=created_after,
                    created_before=created_before,
                    search=search,
                    page=page,
                ),
            )

        if response.status_code == 500:
            raise ServerError(response.text)

        assert response.status_code == 200, response.text

        users_response: UsersResponse = response.json()
        return users_response

    @validate_arguments
    @cache
    async def _get(
        self,
        created_after: Optional[datetime] = None,
        created_before: Optional[datetime] = None,
        search: Optional[str] = None,
    ) -> List[User]:
        response = await self._get_page(
            created_after=created_after,
            created_before=created_before,
            search=search,
        )
        max_pages = response["num_pages"]
        if max_pages > 1:
            coroutines = [
                self._get_page(
                    created_after=created_after,
                    created_before=created_before,
                    search=search,
                    page=page + 2,
                )
                for page in range(max_pages - 1)
            ]
            response_pages = await asyncio.gather(*coroutines)
            responses = (response, *response_pages)
        else:
            responses = (response,)

        return [user for response in responses for user in response["users"]]

    @retry_policy(
        max_retry_count=4,
        max_retry_interval_seconds=10,
        retryable_exceptions=[ServerError],
    )
    @validate_arguments
    @cache
    async def _get_user(
        self,
        user_uuid: str,
    ) -> List[User]:
        async with self._client.session() as session:
            response = await session.get(
                USER_PATH.format(user_uuid=user_uuid),
            )

        if response.status_code == 500:
            raise ServerError(response.text)
        if response.status_code == 404:
            raise UserNotFoundError(response.text)

        assert response.status_code == 200, response.text

        user: User = response.json()
        return [user]


class AsyncUsersResource(BaseUsersResource):
    async def get(
        self,
        *,
        created_after: Optional[datetime] = None,
        created_before: Optional[datetime] = None,
        search: Optional[str] = None,
        user_uuid: Optional[str] = None,
    ) -> List[User]:
        if user_uuid:
            return await self._get_user(
                user_uuid=user_uuid,
            )
        return await self._get(
            created_after=created_after,
            created_before=created_before,
            search=search,
        )


class SyncUsersResource(BaseUsersResource):
    def get(
        self,
        *,
        created_after: Optional[datetime] = None,
        created_before: Optional[datetime] = None,
        search: Optional[str] = None,
        user_uuid: Optional[str] = None,
    ) -> List[User]:
        if user_uuid:
            return self._client.run(
                self._get_user(
                    user_uuid=user_uuid,
                )
            )

        return self._client.run(
            self._get(
                created_after=created_after,
                created_before=created_before,
                search=search,
            )
        )
